import os
import asyncio
import functools
import logging
import logging.config

logger = logging.getLogger(__name__)

import toml

import async_patterns
import async_patterns.protocol
import async_patterns.coro_queue
import ws_storage.impl.as3
import ws_storage.impl.filesystem

class ServerClientProtocol(async_patterns.protocol.Protocol):
    def __init__(self, loop, app):
        logger.debug('{} created'.format(self.__class__.__name__))
        super(ServerClientProtocol, self).__init__(loop)
        logger.debug('super initialized')
        self.app = app

class Application(async_patterns.coro_queue.CoroQueueClass):
    """
    Storage Server

    This class is an async context manager.

    Parameters:

    - dev ``bool``
    - console ``bool``
    - port ``int``
    
    :param loop: event loop
    :param args: dictionary of parameters
    """
    def __init__(self, loop, args):
        self.loop = loop
        self.args = args

        super(Application, self).__init__(loop=loop)
        
    @async_patterns.CoroQueueClass.wrap
    async def write_binary(self, id_, b):
        return (await self.storage.write_binary(id_, b))

    @async_patterns.CoroQueueClass.wrap
    async def read_binary(self, id_):
        res = await self.storage.read_binary(id_)

        if isinstance(res, Exception):
            raise res
    
        logger.debug('read binary id = {} len = {} bytes'.format(repr(id_), len(res)))

        return res

    @async_patterns.CoroQueueClass.wrap
    async def delete(self, id_):
        res = await self.storage.delete(id_)
        
        if isinstance(res, Exception):
            raise res
        
        logger.debug('delete. id = {}'.format(repr(id_)))
        
    @async_patterns.CoroQueueClass.wrap
    async def list_files(self):
        return (await self.storage.list_files())

    async def close(self):
        await super(Application, self).close()
        
        self.server.close()
        await self.server.wait_closed()

    def create_storage(self):
        factory = {
                'FILESYSTEM': ws_storage.impl.filesystem.Storage,
                'AS3': ws_storage.impl.as3.Storage,}
        logger.debug('impl={}'.format(repr(self.conf['implementation'])))
        self.storage = factory[self.conf['implementation']](self.conf, self.loop)
    
    async def __aenter__(self):
        loop, args = self.loop, self.args


        #self.conf = modconf.import_class(
        #        args['conf_mod'], 
        #        'Conf', 
        #        tuple(),
        #        kwargs={
        #            'dev': args.get('dev', False),
        #            'port': args['port'],
        #            'impl': args.get('impl','AS3'),
        #            'conf_dir': args.get('conf_dir', None),
        #            },
        #        folder=args.get('conf_dir', None))
        
        with open('ws_storage.toml') as f:
            self.conf = toml.loads(f.read())
        
        self.create_storage()

        try:
            os.makedirs(os.path.dirname(self.conf['logging']['handlers']['file']['filename']))
        except FileExistsError:
            pass

        logging.config.dictConfig(self.conf['logging'])
    
        coro = loop.create_server(
                functools.partial(ServerClientProtocol, loop, self),
                'localhost', 
                self.conf['port'])
        
        logger.debug('start server')
        self.server = await coro
    
        addr = self.server.sockets[0].getsockname()
        port = addr[1]
        
        logger.debug('serving on {}'.format(addr))
        logger.debug('port = {}'.format(port))
        logger.debug('ws_storage version = {}'.format(ws_storage.__version__))
        
        return self, addr
    
    async def __aexit__(self, exc_type, exc, tb):
        #def stop(loop, app):
        logger.debug('closing')
        # Close the server
        await self.close()

    def __enter__(self):
        return self.loop.run_until_complete(self.__aenter__())

    def __exit__(self, exc_type, exc, tb):
        return self.loop.run_until_complete(self.__aexit__(exc_type, exc, tb))

async def arunserver(future, loop, args):
    async with Application(loop, args) as app:
        await future

def runserver(args):
    loop = asyncio.get_event_loop()
    
    future = loop.create_future()

    try:
        # run forever
        loop.run_until_complete(arunserver(future, loop, args))
    except KeyboardInterrupt:
        logger.debug('got keyboard interrupt')
    except Exception as e:
        logger.error(e)







class FileNotFound(Exception):
    def __init__(self, id_):
        self.id_ = id_
    def __str__(self):
        return 'File not found with id={}'.format(repr(self.id_))
    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__, repr(self.id_))





from ws_storage.tests.conf.simple import Conf as Conf1

class Conf(Conf1):
    @classmethod
    def prepare(cls, mode, conf_dir, impl, port=None):
        Conf1.prepare(mode, conf_dir, impl, port)

        cls.log_console()






__version__ = '0.6b22'

import argparse
import asyncio
import logging
import logging.config
import os
import shutil
import subprocess

import ws_storage.server
import ws_storage.impl.filesystem

logger = logging.getLogger(__name__)

BASE_DIR = os.path.dirname(__file__)

def runserver(args):
    try:
        ws_storage.server.runserver(args)
    except Exception as e:
        logger.exception(str(e))

def install(args):
    # copy systemd file
    shutil.copyfile(
            os.path.join(BASE_DIR, 'ws_storage.service'),
            os.path.join('/lib/systemd/system', 'ws_storage.service'))

    config_dir_dst = '/etc/ws_storage/conf'

    # make etc directory
    try:
        os.makedirs(config_dir_dst)
    except: pass
    
    # copy default config file
    shutil.copyfile(
            os.path.join(BASE_DIR, 'tests/conf/simple.py'),
            os.path.join(config_dir_dst, 'simple.py'))
    shutil.copyfile(
            os.path.join(BASE_DIR, 'tests/conf/simple_console.py'),
            os.path.join(config_dir_dst, 'simple_console.py'))

    p = subprocess.Popen(('systemctl', 'daemon-reload'))
    p.communicate()
    p = subprocess.Popen(('systemctl', 'restart', 'ws_storage.service'))
    p.communicate()

def main(argv):
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
 
    def help_(_):
        parser.print_help()

    parser.set_defaults(func=help_)
   
    parser_runserver = subparsers.add_parser('runserver')
    parser_runserver.add_argument(
            '-d',
            action='store_true',
            help='develop mode',
            )
    parser_runserver.add_argument(
            '--conf_dir',
            default=None,
            help='modconf module directory',
            )
    parser_runserver.add_argument(
            'conf_mod',
            help='modconf module name',
            )
    parser_runserver.add_argument(
            '--impl',
            default=None,
            help='storage implementation',
            )
    parser_runserver.add_argument('--port', type=int)

    parser_runserver.set_defaults(func=runserver)
 
    parser_install = subparsers.add_parser('install')
    parser_install.set_defaults(func=install)

    args = parser.parse_args(argv[1:])
    args.func(vars(args))










import asyncio
import functools
import pickle
import logging

logger = logging.getLogger(__name__)

import ws_storage.packet
import async_patterns.protocol
import ws_storage.exceptions

class Storage(ws_storage.base.Storage, async_patterns.protocol.Protocol):
    """
    This implementation of the Storage interface is also an asyncio Protocol

    :param loop: asyncio event loop

    .. note: This object has a cache and the server storage object will have a cache.
    """
    def __init__(self, loop):
        ws_storage.base.Storage.__init__(self)
        async_patterns.protocol.Protocol.__init__(self, loop)
        self.loop = loop

    async def _read_binary(self, id_):
        p = ws_storage.packet.ReadBinary(id_)
        resp = await self.write(p)

        if isinstance(resp, ws_storage.packet.FileNotFound):
            raise ws_storage.exceptions.FileNotFound(id_)

        return resp.b

    async def _delete(self, id_):
        p = ws_storage.packet.Delete(id_)
        resp = await self.write(p)

        if isinstance(resp, ws_storage.packet.FileNotFound):
            raise ws_storage.exceptions.FileNotFound(id_)

    async def _write_binary(self, id_, b):
        p = ws_storage.packet.WriteBinary(id_, b)
        self.write(p)

    async def list_files(self):
        p = ws_storage.packet.ListFiles()
        resp = await self.write(p)
        logger.debug('resp={}'.format(resp))
        return resp.files

    @classmethod
    async def create(cls, loop, host, port):
        """
        :param loop: asyncio event loop
        :param str host: host
        :param int port: port
        :return: Storage object
        """
        coro = loop.create_connection(
            functools.partial(cls, loop),
            host, 
            port)
        
        _, protocol = await coro
        
        return protocol
        



import os
import functools
import logging

import pytest
import toml

import ws_storage.server
import ws_storage.impl.filesystem
import ws_storage.impl.as3
import ws_storage.impl.client

class_list = [
    ws_storage.impl.filesystem.Storage,
    ws_storage.impl.as3.Storage,
    functools.partial(ws_storage.impl.as3.Storage, use_botocore=True),
    ]

is_client_list = [False, False, False, True]

@pytest.fixture()
async def storage_server(request, event_loop):
    print('fixture storage_server')

    args = {
            'conf_mod': 'ws_storage.tests.conf.simple',
            'conf_dir': os.path.join(os.environ['HOME'], 'config'),
            'dev': True,
            'port': 0,
            }

    async with ws_storage.server.Application(event_loop, args) as (app, addr):
        yield app, addr

@pytest.fixture()
async def storage_client(event_loop, storage_server):
    print('fixture storage_client')
    _, addr = storage_server
    s = await ws_storage.impl.client.Storage.create(event_loop, addr[0], addr[1])
    assert not s.transport.is_closing()
    yield s

@pytest.fixture(params=class_list)
async def storage_default(request, event_loop):
    print('fixture storage_default')
    
    cls = request.param

    args = {
            'dev': True,
            'conf_dir': os.path.join(os.environ['HOME'], 'config'),
            'console': True,
            }
    
    #conf = modconf.import_class(
    #        'ws_storage.tests.conf.simple', 
    #        'Conf', 
    #        tuple(),
    #        args)
    with open('ws_storage.toml') as f:
        conf = toml.loads(f.read())
    
    logging.config.dictConfig(conf['logging'])

    s = cls(conf, event_loop)

    yield s






import struct
import logging

import ws_storage.exceptions

logger = logging.getLogger(__name__)

class Storage(object):
    """
    """
    async def _read_binary(self, id_):
        """
        to be defined by implementation
        """
        pass

    async def _write_binary(self, id_, b):
        """
        to be defined by implementation
        """
        pass

    async def _delete(self, id_):
        raise NotImplemented

    async def list_files(self):
        """
        To be defined by implementation.

        Return a list of the names of all stored files.
        """
        pass

    def __init__(self):
        self._cache = {}

    async def next_id(self):
        logger.debug('next_id')
        try:
            b = await self.read_binary('_counter')
        except ws_storage.exceptions.FileNotFound:
            i = 0
        except Exception as e:
            logger.error(e)
            raise
        else:
            t = struct.unpack('<L', b)
            logger.debug('unpacked = {}'.format(t))
            i = t[0] + 1

        await self.write_binary('_counter', struct.pack('<L', i))
        
        return str(i)

    async def read_binary(self, id_):
        if not id_ in self._cache:
            b = await self._read_binary(id_)
            self._cache[id_] = b
        
        return self._cache[id_]

    async def write_binary(self, id_, b):
        self._cache[id_] = b
        await self._write_binary(id_, b)

    async def delete(self, id_):
        if id_ in self._cache:
            del self._cache[id_]

        await self._delete(id_)






import os
import asyncio
import functools
import pickle
import logging
import logging.config

import pytest

import ws_storage.impl.filesystem
import ws_storage.impl.as3
import ws_storage.impl.client

class Foo: pass

@pytest.mark.asyncio
async def test(storage_default):
    await _test(storage_default)

@pytest.mark.asyncio
async def test_client(storage_client):
    await _test(storage_client)

@pytest.mark.asyncio
async def test_list(storage_default):
    await _test_list(storage_default)

@pytest.mark.asyncio
async def test_list_client(storage_client):
    await _test_list(storage_client)

async def _test(s):
    f = Foo()

    b = pickle.dumps(f)
    
    id_ = await s.next_id()
    
    print('id =',id_)

    await s.write_binary(id_, b)

    b = await s.read_binary(id_)

    f = pickle.loads(b)

    print(f)
    
    await asyncio.sleep(1)

    await s.delete(id_)
    
    with pytest.raises(ws_storage.exceptions.FileNotFound):
        await s.read_binary('doesnt_exist')

    with pytest.raises(ws_storage.exceptions.FileNotFound):
        await s.delete('doesnt_exist')

async def _test_list(s):
    # TODO appears to be bug cause by combination of botocore and aiobotocore
    if isinstance(s, ws_storage.impl.as3.Storage):
        if s._use_botocore:
            with pytest.raises(NotImplementedError):
                await s.list_files()
            return
    await s.list_files()









import asyncio
import botocore.session
import aiobotocore

TEST_BUCKET = 'chuck1-dummy1'
TEST_KEY = 'blah'
CREATE_REGION = 'eu-west-1'
QUERY_REGION = 'us-west-2'

create_client = botocore.session.get_session().create_client('s3', region_name=CREATE_REGION)

def boto_test():
    print('boto_test')
    session = botocore.session.get_session()

    client = session.create_client('s3', region_name=QUERY_REGION)

    resp = client.put_object(
            Bucket=TEST_BUCKET,
            Key=TEST_KEY,
            Body=b'hello')

    print(resp)
    print()

    resp2 = client.get_object(
            Bucket=TEST_BUCKET,
            Key=TEST_KEY)

    print(resp2)
    print()
    return resp, resp2


def aioboto_test():
    print('aioboto_test')
    loop = asyncio.get_event_loop()
    session = aiobotocore.session.get_session()

    async def go():
        async with session.create_client('s3', region_name=QUERY_REGION) as client:
            resp = await client.put_object(
                    Bucket=TEST_BUCKET,
                    Key=TEST_KEY,
                    Body=b'hello')

            print(resp)
            print()

            resp2 = await client.get_object(
                    Bucket=TEST_BUCKET,
                    Key=TEST_KEY)

            print(resp2)
            print()

        return resp, resp2

    loop.run_until_complete(go())

create_client.create_bucket(Bucket=TEST_BUCKET,
                          CreateBucketConfiguration={'LocationConstraint': CREATE_REGION})

try:
    boto_test()
    aioboto_test()
finally:
    create_client.delete_object(Bucket=TEST_BUCKET, Key=TEST_KEY)
    create_client.delete_bucket(Bucket=TEST_BUCKET)




import asyncio
import os
import sys

import botocore.session

sys.path.append(os.path.join(os.environ['HOME'], 'config'))
import aws

conf = aws.Conf

session = botocore.session.get_session()

client = session.create_client(
        's3', 
        region_name=conf.REGION,
        aws_secret_access_key=conf.SECRET_ACCESS_KEY,
        aws_access_key_id=conf.ACCESS_KEY_ID)

resp = client.put_object(
        Bucket=conf.BUCKET,
        Key='test',
        Body=b'hello')

print(resp)
print()

resp = client.get_object(
        Bucket=conf.BUCKET,
        Key='test')

print(resp)
print()


import aiobotocore

loop = asyncio.get_event_loop()

session = aiobotocore.session.get_session()

async def go():

    async with session.create_client(
            's3', 
            region_name=conf.REGION,
            aws_secret_access_key=conf.SECRET_ACCESS_KEY,
            aws_access_key_id=conf.ACCESS_KEY_ID) as client:

        resp = await client.put_object(
                Bucket=conf.BUCKET,
                Key='test',
                Body=b'hello')

        print(resp)
        print()
        
        resp = await client.get_object(
                Bucket=conf.BUCKET,
                Key='test')

        print(resp)
        print()

loop.run_until_complete(go())






import asyncio
import os
import pickle
import logging

import aiobotocore
import botocore
import botocore.session
import botocore.exceptions

import ws_storage.base
import ws_storage.exceptions

logger = logging.getLogger(__name__)

class Storage(ws_storage.base.Storage):
    """
    :param conf: configuration object

    the configuraion must have the following attributes

    - ``AWS_ACCESS_KEY_ID``
    - ``AWS_SECRET_ACCESS_KEY``
    - ``AWS_BUCKET``
    - ``AWS_REGION``

    """
    def __init__(self, conf, loop, use_botocore=False):
        super(Storage, self).__init__()
        self.conf = conf
        self.loop = loop
        
        self._use_botocore = use_botocore
        
        if use_botocore:
            self.session = botocore.session.get_session()
        else:
            self.session_aio = aiobotocore.get_session(loop=loop)

        self.client_kwargs = {
            'region_name': self.conf['aws']['region'],
            'aws_secret_access_key': self.conf['aws']['secret_access_key'],
            'aws_access_key_id': self.conf['aws']['access_key_id'],
            }
        
        logger.debug('client kwargs')
        for k, v in self.client_kwargs.items():
            logger.debug('{:32}{}'.format(k, v))

    async def _write_binary_TEST(self, id_, b):
        logger.debug('write')
        
        client = self.session.create_client(
                's3', **self.client_kwargs)

        logger.debug('client={}'.format(client))
        #await client.create_bucket(
        #        Bucket=self.conf.AWS.BUCKET,)
        
        # upload object to amazon s3
        resp = client.put_object(
                Bucket=self.conf['aws']['bucket'],
                Key=id_,
                Body=b)
        
        logger.debug('resp={}'.format(resp))

    async def _read_binary_TEST(self, id_, endpoint_url=None):
        logger.debug('read')
        
        client = self.session.create_client(
                's3', **self.client_kwargs, endpoint_url=endpoint_url)
        
        logger.debug('client={}'.format(client))
        # get object from s3
        
        try:
            resp = client.get_object(
                    Bucket=self.conf['aws']['bucket'],
                    Key=id_)
        except botocore.exceptions.ClientError as e:
            if e.response['Error']['Code'] == 'NoSuchKey':
                raise ws_storage.exceptions.FileNotFound(id_)
            else:
                raise

        # this will ensure the connection is correctly re-used/closed
        logger.debug('resp={}'.format(resp))
        
        b = resp['Body'].read()
        
        return b
    
    async def _write_binary(self, id_, b):
        logger.debug('write')

        if self._use_botocore:
            return (await self._write_binary_TEST(id_, b))
        
        async with self.session_aio.create_client(
                's3', **self.client_kwargs) as client:
            logger.debug('client={}'.format(client))
            #await client.create_bucket(
            #        Bucket=self.conf.AWS.BUCKET,)
            
            try:
                # upload object to amazon s3
                resp = await client.put_object(
                        Bucket=self.conf['aws']['bucket'],
                        Key=id_,
                        Body=b)
            except Exception as e:
                logger.error(e)
                raise
            
            logger.debug('resp={}'.format(resp))
    
    async def _read_binary(self, id_, endpoint_url=None):
        logger.debug('read')
        
        if self._use_botocore:
            return (await self._read_binary_TEST(id_, endpoint_url))
        
        async with self.session_aio.create_client(
                's3', **self.client_kwargs, endpoint_url=endpoint_url) as client:
            logger.debug('client={}'.format(client))
            # get object from s3
    
            try:
                resp = await client.get_object(
                        Bucket=self.conf['aws']['bucket'],
                        Key=id_)
            except botocore.exceptions.ClientError as e:
                if e.response['Error']['Code'] == 'NoSuchKey':
                    raise ws_storage.exceptions.FileNotFound(id_)
                else:
                    raise

            # this will ensure the connection is correctly re-used/closed
            logger.debug('resp={}'.format(resp))
            async with resp['Body'] as stream:
                b = await stream.read()

            return b

    async def list_files_TEST(self):
        client = self.session.create_client(
                's3', **self.client_kwargs,)

        logger.debug('client={}'.format(client))

        paginator = client.get_paginator('list_objects')

        logger.debug('paginator={}'.format(paginator))
        
        ret = []
        
        p = paginator.paginate(Bucket=self.conf['aws']['bucket'])
        
        logger.debug('paginate={}'.format(p))

        for result in p:
            for c in result.get('Contents', []):
                logger.debug('key={}'.format(c['Key']))
                ret.append(c['Key'])
        
        return ret

    async def list_files(self):
        logger.debug('list files')

        if self._use_botocore:
            return (await self.list_files_TEST())
        
        ret = []
        async with self.session_aio.create_client(
                's3', **self.client_kwargs) as client:
            
            logger.debug('client={}'.format(client))

            # list files
            paginator = client.get_paginator('list_objects')

            logger.debug('paginator={}'.format(paginator))

            async for result in paginator.paginate(Bucket=self.conf['aws']['bucket']):
                logger.debug('result={}'.format(result))
                 
                for c in result.get('Contents', []):
                    logger.debug('c={}'.format(c))
                    ret.append(c)

        return ret

    def _delete_TEST(self, id_):
        logger.debug('delete')
        
        client = self.session.create_client('s3', **self.client_kwargs)
        
        resp = client.delete_object(
                Bucket=self.conf['aws']['bucket'],
                Key=id_)

        logger.debug(resp)

    async def _delete(self, id_):
        logger.debug('delete')

        # use to raise FileNotFound if id doesnt exist
        await self._read_binary(id_)

        if self._use_botocore:
            return self._delete_TEST(id_)

        async with self.session_aio.create_client(
                's3', **self.client_kwargs) as client:
        
            resp = await client.delete_object(
                    Bucket=self.conf['aws']['bucket'],
                    Key=id_)
            logger.debug(resp)






import os
import re
import json
from setuptools import setup

with open('Setup.lock') as f:
    c = json.loads(f.read())

with open(os.path.join(c['name'], '__init__.py')) as f:
    version = re.findall("^__version__ = '(.*)'", f.read())[0]

with open('Pipfile.lock') as f:
    p = json.loads(f.read())

def _install_requires():
    for k, v in p['default'].items():
        if isinstance(v, str):
            yield k + v
        else:
            yield k + v['version'] 

install_requires = list(_install_requires())

kwargs = {
        'name': c['name'],
        'version': version,
        'description': c['description'],
        'url': c['url'],
        'author': c['author'],
        'author_email': c['author_email'],
        'license': c['license'],
        'packages': c.get('packages', []),
        'zip_safe': False,
        'scripts': c.get('scripts',[]),
        'package_data': c.get('package_data',{}),
        'install_requires': install_requires,
        'classifiers': c.get('classifiers', [])
        }

setup(**kwargs)






import asyncio
import os
import pickle
import logging

import ws_storage
import ws_storage.base
import ws_storage.exceptions

logger = logging.getLogger(__name__)

class Storage(ws_storage.base.Storage):
    def __init__(self, conf, loop):
        super(Storage, self).__init__()

        # for the purposes of testing I should create system to create a temp folder for this
        self.dir_ = conf['storage_dir']

    async def _write_binary(self, id_, b):
        logger.debug('_write_binary id={} {} bytes'.format(repr(id_), len(b)))

        try:
            os.makedirs(self.dir_)
        except FileExistsError as e: pass

        filename = os.path.join(self.dir_, str(id_))
        with open(filename, 'wb') as f:
            f.write(b)

    async def _read_binary(self, id_):
        logger.debug('_read_binary({})'.format(id_))

        filename = os.path.join(self.dir_, str(id_))
        
        logger.debug('exists = {}'.format(os.path.exists(filename)))
        
        try:
            with open(filename, 'rb') as f:
                b = f.read()
        except FileNotFoundError:
            raise ws_storage.exceptions.FileNotFound(id_)

        logger.debug('b = {}'.format(repr(b)))

        return b

    async def list_files(self):
        return os.listdir(self.dir_)

    async def _delete(self, id_):
        logger.debug('delete {}'.format(id_))

        filename = os.path.join(self.dir_, str(id_))

        try:
            os.remove(filename)
        except FileNotFoundError:
            raise ws_storage.exceptions.FileNotFound(id_)





import os
import enum

import modconf

class Conf(object):
    class Mode(enum.Enum):
        DEVELOP=1
        DEPLOY=2
        
        @classmethod
        def from_string(cls, s):
            if s == 'DEVELOP':
                return cls.DEVELOP
            elif s == 'DEPLOY':
                return cls.DEPLOY
            else:
                raise Exception()

    @classmethod
    def log_console(cls):
        for l in cls.LOGGING['loggers'].values():
            l['handlers'] = ['console']

    @classmethod
    def prepare(cls, conf_dir=None, impl=None, port=None, **kwargs):

        cls.CONF_DIR = conf_dir

        cls.IMPL = impl if impl is not None else 'AS3'

        cls.DEV = kwargs.get('dev', False)

        if cls.DEV:
            cls.PORT = 10001 if port is None else port
            cls.STORAGE_DIR = '.dev/etc/ws_storage/storage'
            cls.LOG_FILE = '.dev/var/log/ws_storage/debug.log'
        else:
            cls.PORT = 10002 if port is None else port
            cls.STORAGE_DIR = '/etc/ws_storage/storage'
            cls.LOG_FILE = '/var/log/ws_storage/debug.log'

        try:
            os.makedirs(os.path.dirname(cls.LOG_FILE))
        except os.error: pass


        if kwargs.get('console', False):
            cls.log_console()

        try:
            cls.AWS = modconf.import_class('aws', 'Conf', tuple(), folder=cls.CONF_DIR)
        except:
            cls.AWS = type('AWS', tuple(), {})
            cls.AWS.ACCESS_KEY_ID = os.environ['AWS_ACCESS_KEY_ID']
            cls.AWS.SECRET_ACCESS_KEY = os.environ['AWS_SECRET_ACCESS_KEY']
            cls.AWS.REGION = os.environ['AWS_REGION']
            cls.AWS.BUCKET = os.environ['AWS_BUCKET']
        






import asyncio
import logging

import ws_storage
import ws_storage.exceptions

logger = logging.getLogger(__name__)

class Packet(object):
    def __init__(self, response_to=None):
        if response_to is not None:
            self.response_to = response_to

    async def __call__(self, proto):
        pass

class WriteBinary(Packet):
    def __init__(self, id_, o):
        self.id_ = id_
        self.o = o

    async def __call__(self, proto):
        logger.debug('{} call'.format(self.__class__.__name__))

        await proto.app.write_binary(self.id_, self.o)

class ReadBinary(Packet):
    def __init__(self, id_):
        self.id_ = id_

    async def __call__(self, proto):
        logger.debug('{} call'.format(self.__class__.__name__))
        
        try:
            b = await proto.app.read_binary(self.id_)
        except ws_storage.exceptions.FileNotFound:
            logger.info('sending FileNotFound')
            proto.write(FileNotFound(self.id_, self.message_id))
            return
        except Exception as e:
            logger.error('error in ReadBinary call: {}'.format(e))
            proto.write(ServerError(repr(e), self.message_id))
            raise

        logger.debug('read {} bytes'.format(len(b)))

        proto.write(ResponseReadBinary(self.id_, b, self.message_id))

class ServerError(Packet):
    def __init__(self, msg, response_to):
        self.msg = msg
        self.response_to = response_to

class Delete(Packet):
    def __init__(self, id_):
        self.id_ = id_

    async def __call__(self, proto):
        logger.debug('{} call'.format(self.__class__.__name__))
        
        try:
            b = await proto.app.delete(self.id_)
        except ws_storage.exceptions.FileNotFound:
            logger.info('sending FileNotFound')
            proto.write(FileNotFound(self.id_, self.message_id))
            return
        except Exception as e:
            logger.error('error in Delete call: {}'.format(e))
            proto.write(ServerError(repr(e), self.message_id))
            raise

        proto.write(Packet(self.message_id))

class FileNotFound(Packet):
    def __init__(self, id_, response_to):
        self.id_ = id_
        self.response_to = response_to

class ResponseReadBinary(Packet):
   def __init__(self, id_, b, response_to):
        self.id_ = id_
        self.b = b
        self.response_to = response_to

class ListFiles(Packet):
    async def __call__(self, proto):
        logger.debug('{} call'.format(self.__class__.__name__))

        l = await proto.app.list_files()

        logger.debug('l={}'.format(l))

        proto.write(ResponseListFiles(l, self.message_id))

class ResponseListFiles(Packet):
    def __init__(self, files, response_to):
        self.files = files
        self.response_to = response_to


